@@ -1,5 +1,5 @@
 /* digesthmac.c --- Compute DIGEST-MD5 response value.
- * Copyright (C) 2002, 2003, 2004, 2008  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -49,8 +49,6 @@
 #define A2_POST ":00000000000000000000000000000000"
 #define COLON ":"
 #define MD5LEN 16
-#define RESPONSE_LENGTH 32
-#define RSPAUTH_LENGTH RESPONSE_LENGTH
 #define DERIVE_CLIENT_INTEGRITY_KEY_STRING \
   "Digest session key to client-to-server signing key magic constant"
 #define DERIVE_CLIENT_INTEGRITY_KEY_STRING_LEN 65
@@ -78,9 +76,9 @@
    are either NULL, or points to 16 byte arrays that will hold the
    computed keys on output.  Returns 0 on success. */
 int
-digest_md5_hmac (char *output, char secret[MD5LEN], char *nonce,
-		 unsigned long nc, char *cnonce, digest_md5_qop qop,
-		 char *authzid, char *digesturi, int rspauth,
+digest_md5_hmac (char *output, char secret[MD5LEN], const char *nonce,
+		 unsigned long nc, const char *cnonce, digest_md5_qop qop,
+		 const char *authzid, const char *digesturi, int rspauth,
 		 digest_md5_cipher cipher,
 		 char *kic, char *kis, char *kcc, char *kcs)
 {
