@@ -1,5 +1,5 @@
 /* callbacks.c --- Implementation of gsasl callbacks.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -115,7 +115,7 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 	    {
 	      printf ("Using system username `%s' as "
 		      "authentication identity.\n", pw->pw_name);
-	      args_info.authentication_id_arg = strdup (pw->pw_name);
+	      args_info.authentication_id_arg = xstrdup (pw->pw_name);
 	    }
 	  else
 #endif
