@@ -1,5 +1,5 @@
 /* gsasl.c --- Command line interface to libgsasl.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -105,7 +105,7 @@ readln (char **out)
       while (input[j - 1] != '\n' && j < MAX_LINE_LENGTH);
       input[j] = '\0';
 
-      *out = strdup (input);
+      *out = xstrdup (input);
 
       printf ("%s", *out);
     }
@@ -263,16 +263,6 @@ const char version_etc_copyright[] =
      year.  */
   "Copyright %s %d Simon Josefsson.";
 
-static void
-emit_bug_reporting_address (void)
-{
-  /* TRANSLATORS: The placeholder indicates the bug-reporting address
-     for this package.  Please add _another line_ saying
-     "Report translation bugs to <...>\n" with the address for translation
-     bugs (typically your translation team's web or email address).  */
-  printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
-}
-
 static void
 usage (int status)
 {
@@ -296,9 +286,6 @@ main (int argc, char *argv[])
   char *connect_hostname = NULL;
   char *connect_service = NULL;
 #ifdef HAVE_LIBGNUTLS
-  const int kx_prio[] = { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS,
-    GNUTLS_KX_DHE_RSA, GNUTLS_KX_ANON_DH, 0
-  };
   gnutls_anon_client_credentials anoncred;
   gnutls_certificate_credentials x509cred;
 #endif
@@ -326,7 +313,10 @@ main (int argc, char *argv[])
 
   if (args_info.version_given)
     {
-      version_etc (stdout, "gsasl", PACKAGE_NAME, VERSION,
+      const char *p = PACKAGE_NAME;
+      if (strcmp (gsasl_check_version (NULL), PACKAGE_VERSION) != 0)
+	p = PACKAGE_STRING;
+      version_etc (stdout, "gsasl", p, gsasl_check_version (NULL),
 		   "Simon Josefsson", (char *) NULL);
       return EXIT_SUCCESS;
     }
@@ -358,6 +348,7 @@ main (int argc, char *argv[])
       !args_info.client_mechanisms_flag && !args_info.server_mechanisms_flag)
     {
       cmdline_parser_print_help ();
+      emit_bug_reporting_address ();
       return EXIT_SUCCESS;
     }
 
@@ -365,17 +356,17 @@ main (int argc, char *argv[])
     {
       if (strrchr (args_info.connect_arg, ':'))
 	{
-	  connect_hostname = strdup (args_info.connect_arg);
+	  connect_hostname = xstrdup (args_info.connect_arg);
 	  *strrchr (connect_hostname, ':') = '\0';
-	  connect_service = strdup (strrchr (args_info.connect_arg, ':') + 1);
+	  connect_service = xstrdup (strrchr (args_info.connect_arg, ':') + 1);
 	}
       else
 	{
-	  connect_hostname = strdup (args_info.connect_arg);
+	  connect_hostname = xstrdup (args_info.connect_arg);
 	  if (args_info.smtp_flag)
-	    connect_service = strdup ("smtp");
+	    connect_service = xstrdup ("smtp");
 	  else
-	    connect_service = strdup ("imap");
+	    connect_service = xstrdup ("imap");
 	}
     }
   else if (args_info.inputs_num > 0)
@@ -384,9 +375,9 @@ main (int argc, char *argv[])
       if (args_info.inputs_num > 1)
 	connect_service = args_info.inputs[1];
       else if (args_info.smtp_flag)
-	connect_service = strdup ("smtp");
+	connect_service = xstrdup ("smtp");
       else
-	connect_service = strdup ("imap");
+	connect_service = xstrdup ("imap");
     }
 
   if (connect_service && !args_info.smtp_flag && !args_info.imap_flag)
@@ -399,16 +390,16 @@ main (int argc, char *argv[])
     }
 
   if (args_info.imap_flag && !args_info.service_given)
-    args_info.service_arg = strdup ("imap");
+    args_info.service_arg = xstrdup ("imap");
 
   if (args_info.smtp_flag && !args_info.service_given)
-    args_info.service_arg = strdup ("smtp");
+    args_info.service_arg = xstrdup ("smtp");
 
   if (args_info.imap_flag || args_info.smtp_flag)
     args_info.no_client_first_flag = 1;
 
   if (connect_hostname && !args_info.hostname_arg)
-    args_info.hostname_arg = strdup (connect_hostname);
+    args_info.hostname_arg = xstrdup (connect_hostname);
 
   if (!isatty (STDOUT_FILENO))
     setvbuf (stdout, NULL, _IOLBF, BUFSIZ);
@@ -554,16 +545,23 @@ main (int argc, char *argv[])
 	    error (EXIT_FAILURE, 0, _("no X.509 CAs found"));
 	}
 
-      res =
-	gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509cred);
+      res = gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE,
+				    x509cred);
       if (res < 0)
 	error (EXIT_FAILURE, 0, _("setting X.509 GnuTLS credential: %s"),
 	       gnutls_strerror (res));
 
-      res = gnutls_kx_set_priority (session, kx_prio);
-      if (res < 0)
-	error (EXIT_FAILURE, 0, _("setting GnuTLS key exchange priority: %s"),
-	       gnutls_strerror (res));
+      if (args_info.priority_arg)
+	{
+	  const char *err_pos;
+
+	  res = gnutls_priority_set_direct (session, args_info.priority_arg,
+					    &err_pos);
+	  if (res < 0)
+	    error (EXIT_FAILURE, 0,
+		   _("setting GnuTLS cipher priority (%s): %s\n"),
+		   gnutls_strerror (res), err_pos);
+	}
 
       gnutls_transport_set_ptr (session, (gnutls_transport_ptr) sockfd);
 
