@@ -1,5 +1,5 @@
 /* external.c --- Test the EXTERNAL mechanism.
- * Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -31,8 +31,8 @@
 
 static const struct
 {
-  char *sendauthzid;
-  char *recvauthzid;
+  const char *sendauthzid;
+  const char *recvauthzid;
   int clientrc;
   int callbackrc;
   int serverrc;
@@ -52,8 +52,8 @@ static const struct
 static int
 callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 {
-  static int c = 0;
-  static int s = 0;
+  static unsigned c = 0;
+  static unsigned s = 0;
   int rc = GSASL_NO_CALLBACK;
 
   c = c % sizeof (tv) / sizeof (tv[0]);
@@ -101,7 +101,7 @@ doit (void)
 
   for (i = 0; i < 2 * (sizeof (tv) / sizeof (tv[0])); i++)
     {
-      int n = i % sizeof (tv) / sizeof (tv[0]);
+      size_t n = i % sizeof (tv) / sizeof (tv[0]);
 
       res = gsasl_server_start (ctx, "EXTERNAL", &server);
       if (res != GSASL_OK)
@@ -138,7 +138,7 @@ doit (void)
 	  return;
 	}
       if (s1)
-	free (s1);
+	gsasl_free (s1);
 
       if (debug)
 	{
@@ -150,7 +150,7 @@ doit (void)
 
       res = gsasl_step (server, s2, s2len, &s1, &s1len);
       if (s2)
-	free (s2);
+	gsasl_free (s2);
       if (res != tv[n].serverrc)
 	{
 	  fail ("gsasl_step server2 (%d):\n%s\n", res, gsasl_strerror (res));
@@ -171,7 +171,7 @@ doit (void)
 	}
 
       if (s1)
-	free (s1);
+	gsasl_free (s1);
 
       gsasl_finish (client);
       gsasl_finish (server);
